; tinyfrac.asm
;
; teeny program displays the Mandelbrot set.
;
; written on Sun  03-26-1995  by Ed Beroset (Fidonet 1:3641/1.250)
;
; This program was based on a program by Frank Hommers, later optimized
; for size by Mikko Hyvarinen and posted in Fidonet's 80XXX echo.
;
; This new version has many new features and was based on my own
; optimization of Hyvarinen's version.  Some features:
;
; pan     using the arrow keys, one can navigate the fractal.
;
;               Home  Up  PgUp
;               Left      Right   correspond to 8 obvious directions
;               End   Dn  PgDn
;
; zoom    there are now ten levels of magnification available.  If the
;         program is assembled with FEATURES defined, the number
;         corresponding to the zoom level (0-9, zero is most zoomed in)
;         is displayed in the upper left hand corner of the screen just
;         before each new fractal is drawn.  The gray '+' key zooms out,
;         the gray '-' key zooms in.
;
; beep    the program will beep at the completion of each fractal
;         drawing or if the user attempts to zoom past either limit.
;
; mode    if the program is assembled with MODECHANGE defined, the
;         will change to the next video mode if the 'v' key is pressed.
;         This is handy because drawing fractals at high resolution can
;         be very timeconsuming.  The user can find an interesting spot
;         in a low res mode and then change to a high res mode to see it
;         more fully rendered.
;
; size    this whole project was started off as a size optimization
;         exercise, so there have been some rather ugly tradeoffs to
;         sacrifice speed for size.
;
; 8086    yes, it runs on an 8086 although only if you leave out either
;         the FEATURES option or the MODECHANGE option and it would be
;         slower and more painful than oral surgery.
;
; cost    there IS such a thing as a free lunch!  This code is hereby
;         released to the public domain by the author.
;
;
; to assemble &amp; link:
;   TASM /m2 tinyfrac       (assemble using two pass mode if required)
;   TLINK /Tdc tinyfrac     (link Target platform is DOS, COM file)
;
;

PIXWIDTH    equ 512
PIXHEIGHT   equ 256

ZOOMLIMIT   equ  13       ; can change to up to 13 for extended zoom in

; feel free to experiment with the following constants:

DELTA       equ 200       ; the unit of pan movement in pixels
THRESHOLD   equ  7        ; must be in the range of (0,255)
STARTSCALE  equ  6        ; a number from 0 to ZOOMLIMIT, inclusive
CHAR_COLOR  equ 0fh       ; white on black background (for PRINTZOOM feature)



; ************************************************************
;
;   Menuet header


use32

                  org     0x0

                  db      'MENUET01'
                  dd      0x01
                  dd      START
                  dd      I_END
                  dd      0x62000
                  dd      0x1000
                  dd      0,0

include 'lang.inc'
include '..\..\macros.inc'
purge mov       ; decrease kpack'ed size

STARTX_ =  200
STARTY_ =  -136

scaleaddx = 200
scaleaddy = -136

START:

red:
draw_window:

      mov  eax,12
      mov  ebx,1
      mcall

      xor  eax,eax
      mov  ebx,50*65536+PIXWIDTH+8
      mov  ecx,100*65536+PIXHEIGHT+25
      mov  edx,0x13334455
      mov  edi,title
      mcall

      mov  al,8
      mov  ebx,290*65536+112
      mov  ecx,5*65536+12
      mov  edx,2
      mov  esi,0x808080
      mcall

      mov  ebx,420*65536+12
;      mov  ecx,5*65536+12
      inc  edx
      mov  esi,0xa00000
      ;mov  eax,8
    newcolor:
      mcall
      add  ebx,13*65536
      inc  edx
      shr  esi,8
      jnz  newcolor

      mov  al,4
      mov  ebx,300*65536+8
      mov  ecx,0x80ffffff
      mov  edx,button_txt
      mcall

      mov  al,12
      mov  ebx,2
      mcall
redraw_fractal:
        jmp     draw_fractal

still:
        mov  eax,10
        mcall

        dec  eax
        jz   red
        dec  eax
        jz   key

      button:
        mov  al,17
        mcall
        shr  eax,8

        dec  eax
        jne  no_close
        dec  eax
        mcall
      no_close:

        mov  ebx,eax
        dec  eax
        jne  no_bgr


        mov  al,15   ; bgr 512 x 256
;        mov  ebx,1
        mov  ecx,512
        mov  edx,256
        mcall

;        mov  eax,15
        mov  bl,5
        mov  ecx,0x1000
        mov  edx,0
        mov  esi,512*3*256
        mcall

;        mov  eax,15
        mov  bl,3
        mcall
        jmp  still

      no_bgr:

; color set:
; id 3, eax=1 - 0x10
; id 4, eax=2 - 0x8
; id 5, eax=3 - 0x0
        xor     al, 3
        shl     al, 3
        mov     [shlc], al
        jmp     draw_fractal

      key:
        mov  al,2
        mcall
        mov  al,ah

     mov  ebp,0x08080808
     mov  esi,(256/8)*5
        cmp  al,'e'
        je   color_cycle
        neg  ebp
        dec  esi
        cmp  al,'r'
        jne  no_color_cycle
color_cycle:

   newcycle:
     mov  edi,0x1000
   newpix:
     add  [edi],ebp
     scasd
     cmp  edi,0x1000+512*256*3
     jb   newpix
     call put_image
     mov  al,5
     mov  ebx,1
     mcall
     dec  esi
     jnz  newcycle
     jmp  draw_fractal

no_color_cycle:
        mov  ebx,[STARTX]
        mov  ecx,[STARTY]
        cmp  al,'q'
        jne  no_in
        inc  byte [scale]
        sub  ebx,scaleaddx/2
        add  ebx,ebx
        sub  ecx,scaleaddy/2
        add  ecx,ecx
      no_in:

        cmp  al,'w'
        jne  no_out
        dec  byte [scale]
        shr  ebx,1
        add  ebx,scaleaddx/2
        shr  ecx,1
        add  ecx,scaleaddy/2
      no_out:

        sub     al, 128+48
        jnz     no_left
        sub     ebx, 100
no_left:
        dec     al
        jnz     no_down
        add     ecx, 100
no_down:
        dec     al
        jnz     no_up
        sub     ecx, 100
no_up:
        dec     al
        jnz     no_right
        add     ebx, 100
no_right:

        mov  [STARTX],ebx
        mov  [STARTY],ecx


; **********************************************************************
;
;    Tinyfrac
;


draw_fractal:

        mov     eax,4
        mov     ebx,10*65536+30
        mov     ecx,0x80ffffff
        mov     edx,calc
        mcall

        mov     ebp, 12345678h
label STARTX dword at $-4
store dword STARTX_ at $-4
        mov     esi, 12345678h
label STARTY dword at $-4
store dword STARTY_ at $-4


;       This routine is the fractal drawing engine.  It has been
;       optimized for size, sacrificing speed.
        mov     edi, 0x1000

@@CalcRow:

        mov     ecx, PIXWIDTH   ; width of screen in pixels
        sub     ebp,ecx         ;
@@CalcPixel:
        push    ecx             ; save the column counter on stack
        xor     ecx, ecx        ; clear out color loop counter
        xor     ebx, ebx        ; zero i coefficient
        xor     edx, edx        ; zero j coefficient
@@CycleColors:
        push    edx             ; save j value for later
        mov     eax, ebx        ; eax = i
        sub     eax, edx        ; eax = i - j
        add     edx, ebx        ; edx = i + j
        call    Shifty          ; eax = ((i+j)*(i-j)) shifted right
        pop     edx             ; retrieve our saved value for j
        add     eax,ebp         ; account for base offset...
        cmp     ah,THRESHOLD    ; Q: is i &gt; THRESHOLD * 256?
        xchg    ebx,eax         ; now swap new i with old i
        jg      @@draw          ; Y: draw this pixel
        call    Shifty          ; now dx:ax = old i * j
        lea     edx, [eax*2+esi]
        inc     cl              ; increment color
        jnz     @@CycleColors   ; keep going until we're done
@@draw:
        xchg    eax, ecx        ; mov color into al
        pop     ecx             ; retrieve our column counter

        shl     al, 3
        shl     eax, 0
shlc = $-1
        stosd
        dec     edi

        inc     ebp
        loop    @@CalcPixel
        inc     esi
        cmp     edi, 0x1000 + PIXWIDTH*PIXHEIGHT*3
        jb      @@CalcRow
        push    still

put_image:

        mov  eax,7
        mov  ebx,0x1000
        mov  ecx,512*65536+256
        mov  edx,4*65536+21
        mcall

        ret


;****************************************************************************
;
;       This routine multiplies AX by DX and shifts the result (in
;       DX:AX) to the right by scale bits (or scale+1 bits if CY is
;       set).  The resulting value is left in AX.  DX is destroyed.
;
;****************************************************************************

Shifty:
        imul    dx             ; do the multiply

        xchg    eax,edx         ;
        shl     eax,16          ; put hi part in hi 16 bits
        xchg    ax,dx
        shr     eax,0
scale = $-1
store byte STARTSCALE at scale

        ret                     ;


; ***************************************************************
;
;     DATA AREA
;


title      db 'Tinyfrac - MOVE: ARROWS, ZOOM Q/W, CYCLE: E/R',0
button_txt  db 'Set as wallpaper',0

calc        db 'CALCULATING',0

I_END:
